# Generated by Django 3.2.4 on 2021-09-22 14:59

import bx_django_utils.data_types.gtin.model_fields
import bx_django_utils.data_types.gtin.validators
import bx_django_utils.models.color_field
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ColorFieldTestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('required_color', bx_django_utils.models.color_field.ColorModelField(max_length=7)),
                ('optional_color', bx_django_utils.models.color_field.ColorModelField(blank=True, max_length=7, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='GtinFieldTestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('default_gtin', bx_django_utils.data_types.gtin.model_fields.GtinModelField(max_length=14, validators=[bx_django_utils.data_types.gtin.validators.GtinValidator(accepted_length=(12, 13, 14))], verbose_name='GTIN')),
                ('all_gtin', bx_django_utils.data_types.gtin.model_fields.GtinModelField(blank=True, max_length=14, validators=[bx_django_utils.data_types.gtin.validators.GtinValidator(accepted_length=(12, 13, 14))], verbose_name='GTIN')),
                ('ean13', bx_django_utils.data_types.gtin.model_fields.GtinModelField(blank=True, max_length=14, null=True, validators=[bx_django_utils.data_types.gtin.validators.GtinValidator(accepted_length=(12, 13, 14))], verbose_name='GTIN')),
            ],
        ),
        migrations.CreateModel(
            name='TimetrackingTestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('create_dt', models.DateTimeField(blank=True, editable=False, help_text='ModelTimetrackingMixin.create_dt.help_text', null=True, verbose_name='ModelTimetrackingMixin.create_dt.verbose_name')),
                ('update_dt', models.DateTimeField(blank=True, editable=False, help_text='ModelTimetrackingMixin.update_dt.help_text', null=True, verbose_name='ModelTimetrackingMixin.update_dt.verbose_name')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CreateOrUpdateTestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('create_dt', models.DateTimeField(blank=True, editable=False, help_text='ModelTimetrackingMixin.create_dt.help_text', null=True, verbose_name='ModelTimetrackingMixin.create_dt.verbose_name')),
                ('update_dt', models.DateTimeField(blank=True, editable=False, help_text='ModelTimetrackingMixin.update_dt.help_text', null=True, verbose_name='ModelTimetrackingMixin.update_dt.verbose_name')),
                ('name', models.CharField(max_length=64)),
                ('slug', models.SlugField(max_length=64)),
                ('blank_field', models.CharField(blank=True, max_length=64)),
                ('null_field', models.CharField(blank=True, max_length=64, null=True)),
                ('many2one_rel', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='test_app.timetrackingtestmodel')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
